import { Component } from 'preact';
import { route } from 'preact-router';
import style from '../routes/snippet/style';
import axios from 'axios';
import SyntaxHighlighter from 'react-syntax-highlighter';
import { atomOneDark } from 'react-syntax-highlighter/dist/esm/styles/hljs';
import { getCookie, formatQuestionText } from './utils';

export default class ComprehensionSnippet extends Component {
    constructor(props) {
        super();
        this.state = {
			fid: -1,
			snippet: '',
			comment: {},
			relevantSummaries: [],
			questionAnswer: '',
			snippetIdx: -1,
			question: '',
			questionAnswer: ''
		};
        this.handleTextChange = this.handleTextChange.bind(this);
		this.submitQuestion = this.submitQuestion.bind(this);
		this.incrementSnippet = this.incrementSnippet.bind(this);
	}

	componentDidMount() {
		const sessionId = getCookie('session_id');
		if (sessionId) {
			axios.post('/api/userStatus', { sessionId }).then(result => {
				if (result.data && result.data.success) {
					const { snippetIdx, response, section, question } = result.data;
					const questionAnswer = response ? response.answer : '';
					const { fid, snippet, snippetName, comment, relevantSummaries } = result.data.data;
					this.setState({ fid, snippet, snippetName, snippetIdx, comment, relevantSummaries, question, questionAnswer, section });
				} else {
					route('/', true);
				}
			});
		} else {
			route('/', true);
		}
	}
	
	incrementSnippet() {
		const sessionId = getCookie('session_id');
		axios.post('/api/nextQuestion', { sessionId }).then(result => {
			if (result.data && result.data.success) {
				const { snippetIdx, response, section } = result.data;
				if (section != this.props.section) {
					this.props.reloadState();
				} else {
					const questionAnswer = response ? response.answer : '';
					const { question } = result.data;
					const { fid, snippet, snippetName, comment, relevantSummaries } = result.data.data;
					this.setState({ fid, snippet, snippetName, snippetIdx, comment, relevantSummaries, question, questionAnswer });
				}
			}
		});
	}
	
	submitQuestion(e) {
		e.preventDefault();
		const { fid, question, questionAnswer, snippetIdx } = this.state;
		const sessionId = getCookie('session_id');

		const postData = {
			fid,
			questionKey: question.data,
			answer: questionAnswer,
			sessionId
		}
		return axios.post(`/api/snippet/${this.props.section}/${snippetIdx}`, postData).then(result => {
			if (result.data && result.data.success) {
				this.setState({ questionAnswer: '' });
				this.incrementSnippet();
			} else {
				alert('Something went wrong.');
			}
		});
	}

    handleTextChange(e) {
		this.setState({ questionAnswer: e.target.value });
    }

    render() {
        const {
			snippet,
			snippetName,
			comment,
			relevantSummaries,
			snippetIdx,
            question,
            decrementSnippet
		} = this.state;
		let blocks = [];
		if (question && question.data) {
			blocks = question.data.split('```');
		}
		const formattedQuestionText = blocks.map((block, idx) => 
			(idx % 2 == 0 ? block : (<span class={style.inlineCode}>{block}</span>)));

		return (
			<div class={style.profile}>
				<h1>Snippet {snippetIdx + 1}</h1>
				<p>
					Spend some time reading through the below snippet, then answer the following questions.
					Note, please refer to the right-hand section of the window for information regarding
					relevant methods used in the code.
				</p>
				<div class={style.mainContentWrapper}>
					<div class={style.code}>
						<SyntaxHighlighter
							language="java"
							style={atomOneDark}
							showLineNumbers={true}
						>{snippet}</SyntaxHighlighter>
					</div>
					<div class={style.relevantCommentsWrapper}>
						<p class={style.comment}>
							<span class={style.methodName}>{snippetName}</span><br />
							{comment.text}
						</p>
						{ (Object.keys(relevantSummaries).length) ? (
								<div>
									<h2 style={{marginTop: 10}}>Relevant Comments</h2>
									<div class={style.relevantComments}>
										{
											Object.keys(relevantSummaries).map((key, idx) => {
												return (<div key={key + '|' + idx} class={style.commentBox}>
													<p class={style.methodName}>{key}</p>
													<p>{relevantSummaries[key].text}</p>
												</div>)
											})
										}
									</div>
								</div>
							) : (<p style={{marginTop: 10}}>No relevant comments available.</p>) }
					</div>
				</div>
				<form onSubmit={this.submitQuestion}>
					<div class={style.questionWrapper}>
						<h2>{formattedQuestionText}</h2>
						{ question.type == 'question' ?
							(<textarea required class={style.question} value={this.state.questionAnswer} onChange={this.handleTextChange}></textarea>) :
							question.type == 'radio' ? question.options.map(elt => (
								<div><input type='radio' name='question' value={elt.value} required={true} onChange={this.handleTextChange}></input>{elt.label}<br/></div>
							)) : 'This question is corrupted. Please contact seancs@umich.edu.' }
					</div>
					<button type="submit" class={style.button}>Next Question</button>
				</form>
			</div>
		);
    }
}