import { Component } from 'preact';
import { route } from 'preact-router';
import style from './style';
import axios from 'axios';
import SyntaxHighlighter from 'react-syntax-highlighter';
import { atomOneDark } from 'react-syntax-highlighter/dist/esm/styles/hljs';
import { getCookie } from '../../components/utils';
import ComprehensionSnippet from '../../components/comprehension';
import TaskBasedSnippet from '../../components/writeMethod';
import CompletedPage from '../../components/completed';

export default class Snippet extends Component {
	constructor() {
		super();
		this.state = {
			section: ''
		};
		this.reloadState = this.reloadState.bind(this);
	}

	reloadState() {
		const sessionId = getCookie('session_id');
		if (sessionId) {
			axios.post('/api/userStatus', { sessionId }).then(result => {
				if (result.data && result.data.success) {
					const { section } = result.data;
					console.log('new section:', section);
					this.setState({ section });
				} else {
					route('/', true);
				}
			});
		} else {
			route('/', true);
		}
	}

	componentDidMount() {
		this.reloadState();
	}

	// Note: `user` comes from the URL, courtesy of our router
	render() {
		const { section } = this.state;
		console.log('section:', section);
		if (section == 'comprehension') {
			return <ComprehensionSnippet
				section='comprehension'
				reloadState={this.reloadState}
			/>
		} else if (section == 'methodWrite' || section == 'repair') {
			return <TaskBasedSnippet
				section={section}
				reloadState={this.reloadState}
			/>
		} else if (section == 'completed') {
			return <CompletedPage />
		} else if (section == 'completed_illegitimate') {
			return (<p>You have already completed this survey. Thank you for your participation.</p>);
		}
	}
}
